/*
 *  abs2rel.c
 *
 *  Usage:
 *     abs2rel path1 path2
 *
 *  Purpose:
 *     convert absolute path1 to relative from path2
 *
 *  Example:
 *     abs2rel /var/ftp/pub/data/qwe.txt /var/ftp/pub/indexes
 *  Result ==> ../data/qwe.txt
 *
 */

#include <unistd.h>   /* write */
#include <stdlib.h>   /* exit */
#include <string.h>   /* strlen, strchr, ... */

#ifndef MAXPATH
#  define MAXPATH 1024
#endif

#define SLASH     '/'
#define SLASH_STR "/"

int echo(const char *str)
{
        write(1, str, strlen(str));
        write(1, "\n", 1);
}

void usage(const char *error_msg)
{
        echo("abs2rel ver.0.1. Written at Mar 2003 by <evseev@ucvt.ru>");
        echo("Usage: abs2rel path1 path2");
        exit(1);
}

int absolute2relative(char result[MAXPATH], const char *destpath,
        const char *srcpath)
{
        const char *p1 = destpath, *q1;
        const char *p2 =  srcpath, *q2;

        result[0] = 0;

        while(1) {
                q1 = strchr(p1, SLASH);
                q2 = strchr(p2, SLASH);

                if (!q1 || !q2 || (q1-p1 != q2-p2) || memcmp(p1, p2, q1-p1))
                        break;

                p1 = q1 + 1;
                p2 = q2 + 1;
        }

        while ((q2 = strchr(p2, SLASH)) != NULL) {
                strcat(result, ".." SLASH_STR);
                p2 = q2 + 1;
        }

        strcat(result, p1);

        return strlen(result);
}

int main(int argc, char *argv[])
{
        char relpath[MAXPATH];

        if (argc != 3)
                usage(NULL);

        if (absolute2relative(relpath, argv[1], argv[2]) < 0)
                return 1;

        echo(relpath);

        return 0;
}

/* EOF */
